from pydantic import BaseSettings, validator
from nonebot import get_driver
from nonebot.log import logger
from pydantic.fields import ModelField
class Config(BaseSettings):
    novelai_tag:str=""
    novelai_cd:int=60
    novelai_limit:bool=True
    
    @validator("novelai_cd")
    def non_negative(cls, v: int, field: ModelField):
        if v < 1:
            return field.default
        return v
    
    class Config:
        extra = "ignore"
config=Config(**get_driver().config.dict())
logger.debug(f"加载config完成"+str(config))


from nonebot import on_command, get_bot
from nonebot.params import CommandArg
from nonebot.adapters.onebot.v11 import MessageSegment, Message, GroupMessageEvent
from .data import *
import aiohttp
import base64
import time
from pathlib import Path
from .config import config
from nonebot.log import logger

path = Path("data/novelai/output").resolve()
txt2pix = on_command(".aidraw", aliases={"文本生图", "咏唱"})

cd = {}
gennerating = False
limit_list = []


@txt2pix.handle()
async def txt2pix_handle(event: GroupMessageEvent, args: Message = CommandArg()):
    nowtime = time.time()
    if (nowtime-cd.get(event.user_id, 0)) < config.novelai_cd:
        await txt2pix.finish(f"你冲的太快啦，请休息一下吧")
    else:
        cd[event.user_id] = nowtime
    message_raw = args.extract_plain_text().replace("，", ",").split("-")
    map = [640, 640]
    input = ""
    seed_raw = None
    for i in message_raw:
        match i:
            case "square" | "s" | "S":
                map = [640, 640]
            case "portrait" | "p" | "P":
                map = [768, 512]
            case 'landscape' | "l" | "L":
                map = [768, 512]
            case _:
                if i.isdigit():
                    seed_raw = int(i)
                else:
                    input += i
    if not input:
        await txt2pix.finish(f"请描述你想要生成的角色特征(使用英文Tag,代码内已包含优化TAG)")
    seed = seed_raw or int(time.time())
    if config.novelai_limit:
        limit_list.append((event.group_id, event.user_id, map, seed, input))
        await txt2pix.send(f"排队中，你的前面还有{len(limit_list)-1}人")
        if not gennerating:
            await run_txt2pix(None)
    else:
        await run_txt2pix((event.group_id, event.user_id, map, seed, input))


async def run_txt2pix(x):
    groupid, userid, map, seed, input = x or limit_list.pop(0)
    if x == None and not gennerating:
        gennerating = True
        logger.debug(f"novelai开始工作")
    bot = get_bot()
    logger.debug(f"novelai开始生成"+input+f",剩余{len(limit_list)}人")
    async with aiohttp.ClientSession("https://api.novelai.net/", headers=header) as session:
        async with session.post("/ai/generate-image", json=txt2pix_body(seed, input, map)) as resp:
            if resp.status != 201:
                await bot.call_api(
                    "send_group_msg", **{
                        "message": f"生成失败，错误代码为"+str(resp.status),
                        "group_id": groupid,
                    },
                )
            img = await resp.text()
            img_bytes = img.split("data:")[1]
            img = base64.b64decode(img_bytes)
            if len(input) > 100:
                input = input[:100]
            img_name = input+str(seed)+".png"
            path.mkdir(parents=True, exist_ok=True)
            with open(path/img_name, "wb") as f:
                f.write(img)
            message = MessageSegment.at(
                userid)+str(seed)+MessageSegment.image("base64://"+img_bytes)
            await bot.call_api(
                "send_group_msg", **{
                    "message": message,
                    "group_id": groupid,
                },
            )
            logger.debug(f"novelai生成完成"+input)
    if x == None:
        if len(limit_list)>0:
            await run_txt2pix(None)
        else:
            gennerating = False
            logger.debug(f"novelai开始休息")




from nonebot import get_driver
from .config import config
lowQuality = 'lowres, bad anatomy, bad hands, text, error, missing fingers, extra digit, fewer digits, cropped, worst quality, low quality, normal quality, jpeg artifacts, signature, watermark, username, blurry,nsfw,'
basetag="masterpiece,extremely detailed,best quality,"
token=get_driver().config.novelai_token
header={
        "authorization":'Bearer '+token,
        ":authority": 'api.novelai.net',
        ":path":'/ai/generate-image',
        'content-type': 'application/json',
        "referer":'https://novelai.net/',
        'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/106.0.0.0 Safari/537.36',
    }
def txt2pix_body(seed,input,map):
    return {
            "input":input+basetag+config.novelai_tag,
            "model":"safe-diffusion",
            "parameters":{
                "width":map[1],
                "height":map[0],
                "scale":11,
                "sampler":"k_euler_ancestral",
                "steps":28,
                "seed":seed,
                "n_samples":1,
                "ucPreset":0,
                "uc":lowQuality,
                }
        }



# -*- coding: utf-8 -*-
from setuptools import setup

import codecs

with codecs.open('README.md', encoding="utf-8") as fp:
    long_description = fp.read()
INSTALL_REQUIRES = [
    'aiohttp>=3.8.3',
    'nonebot-adapter-onebot>=2.1.3',
    'nonebot2>=2.0.0rc1',
]

setup_kwargs = {
    'name': 'nonebot-plugin-novelai',
    'version': '0.2.4',
    'description': '基于nonebot2的novelai绘图插件',
    'long_description': long_description,
    'license': 'MIT',
    'author': '',
    'author_email': 'sena-nana <851183156@qq.com>',
    'maintainer': None,
    'maintainer_email': None,
    'url': '',
    'packages': [
        'nonebot-plugin-novelai',
    ],
    'package_data': {'': ['*']},
    'long_description_content_type': 'text/markdown',
    'install_requires': INSTALL_REQUIRES,
    'python_requires': '>=3.10',

}


setup(**setup_kwargs)
